"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.guard = void 0;
const symbols_1 = require("./symbols");
const utils_1 = require("./utils");
function guard(name, fn, handleOptions, defaultOptions) {
    function getFnWithErrors(options) {
        return (value, path = []) => {
            const res = fn(value, options);
            if (!(0, utils_1.isValid)(res)) {
                if (value === undefined && options.default !== undefined) {
                    return { valid: true, data: options.default, errors: [] };
                }
                return {
                    valid: false,
                    data: undefined,
                    errors: [
                        {
                            message: `vality.${name}.base`,
                            path,
                            options,
                            value,
                        },
                    ],
                };
            }
            if (options.validate) {
                if (!options.validate(res, options)) {
                    return {
                        valid: false,
                        data: undefined,
                        errors: [
                            {
                                message: `vality.${name}.custom`,
                                path,
                                options,
                                value,
                            },
                        ],
                    };
                }
            }
            let data = res;
            if (options.transform) {
                data = options.transform(data);
            }
            if (handleOptions === undefined)
                return { valid: true, data: data, errors: [] };
            const optionsWithDefault = Object.assign(Object.assign({}, defaultOptions), options);
            const keysWithError = Object.keys(optionsWithDefault).filter(k => k !== "transform" &&
                k !== "validate" &&
                k !== "default" &&
                handleOptions[k] !== undefined &&
                !handleOptions[k](res, optionsWithDefault[k], options));
            if (keysWithError.length === 0) {
                return {
                    valid: true,
                    data,
                    errors: [],
                };
            }
            return {
                valid: false,
                data: undefined,
                errors: keysWithError.map(k => ({
                    message: k in options ? `vality.${name}.options.${k}` : `vality.${name}.base`,
                    options,
                    path,
                    value,
                })),
            };
        };
    }
    return Object.assign((options) => {
        return {
            [symbols_1._validate]: (val, path, parent) => {
                if (typeof options === "function")
                    options = options(parent);
                return getFnWithErrors(Object.assign({}, options))(val, path);
            },
            [symbols_1._type]: undefined,
        };
    }, {
        [symbols_1._validate]: getFnWithErrors({}),
        [symbols_1._type]: undefined,
    });
}
exports.guard = guard;
